@@ -1,12 +1,15 @@
+from __future__ import annotations
+
 from collections import defaultdict
 from dataclasses import dataclass
 from functools import lru_cache
+from typing import Any, cast
 
 from django.apps import apps
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import FieldError
 from django.db import DEFAULT_DB_ALIAS, models
-from django.db.models import Q, Subquery
+from django.db.models import Model, Q, Subquery
 
 
 @dataclass(frozen=True)
@@ -15,11 +18,11 @@ class ParentLinkInfo:
     Information about a parent table link in a polymorphic model.
     """
 
-    model: models.Model
-    link: models.Field
+    model: type[models.Model]
+    link: models.Field[Any, Any]
 
 
-def reset_polymorphic_ctype(*models, **filters):
+def reset_polymorphic_ctype(*models: type[models.Model], **filters: Any) -> None:
     """
     Set the polymorphic content-type ID field to the proper model
     Sort the ``*models`` from base class to descending class,
@@ -31,18 +34,18 @@ def reset_polymorphic_ctype(*models, **filters):
     using = filters.pop("using", DEFAULT_DB_ALIAS)
     ignore_existing = filters.pop("ignore_existing", False)
 
-    models = sort_by_subclass(*models)
+    models_list: list[type[Model]] = sort_by_subclass(*models)
     if ignore_existing:
         # When excluding models, make sure we don't ignore the models we
         # just assigned the an content type to. hence, start with child first.
-        models = reversed(models)
+        models_list = list(reversed(models_list))
 
-    for new_model in models:
+    for new_model in models_list:
         new_ct = ContentType.objects.db_manager(using).get_for_model(
             new_model, for_concrete_model=False
         )
 
-        qs = new_model.objects.db_manager(using)
+        qs = new_model.objects.db_manager(using)  # type: ignore[attr-defined]
         if ignore_existing:
             qs = qs.filter(polymorphic_ctype__isnull=True)
         if filters:
@@ -50,7 +53,7 @@ def reset_polymorphic_ctype(*models, **filters):
         qs.update(polymorphic_ctype=new_ct)
 
 
-def _compare_mro(cls1, cls2):
+def _compare_mro(cls1: type, cls2: type) -> int:
     if cls1 is cls2:
         return 0
 
@@ -67,7 +70,7 @@ def _compare_mro(cls1, cls2):
     return (index1 > index2) - (index1 < index2)  # python 3 compatible cmp.
 
 
-def sort_by_subclass(*classes):
+def sort_by_subclass(*classes: type[models.Model]) -> list[type[models.Model]]:
     """
     Sort a series of models by their inheritance order.
     """
@@ -77,25 +80,29 @@ def sort_by_subclass(*classes):
 
 
 @lru_cache(maxsize=None)
-def get_base_polymorphic_model(ChildModel, allow_abstract=False):
+def get_base_polymorphic_model(
+    ChildModel: type[models.Model], allow_abstract: bool = False
+) -> type[models.Model] | None:
     """
     First the first concrete model in the inheritance chain that inherited from the
     PolymorphicModel.
     """
     from polymorphic.models import PolymorphicModel
 
-    for Model in reversed(ChildModel.mro()):
+    for base_class in reversed(ChildModel.mro()):
         if (
-            issubclass(Model, PolymorphicModel)
-            and Model is not PolymorphicModel
-            and (allow_abstract or not Model._meta.abstract)
+            issubclass(base_class, PolymorphicModel)
+            and base_class is not PolymorphicModel
+            and (allow_abstract or not base_class._meta.abstract)
         ):
-            return Model
+            return base_class
     return None
 
 
 @lru_cache(maxsize=None)
-def route_to_ancestor(model_class, ancestor_model):
+def route_to_ancestor(
+    model_class: type[models.Model], ancestor_model: type[models.Model]
+) -> list[ParentLinkInfo]:
     """
     Returns the first (highest mro precedence - depth first on parents) model
     inheritance route to the given ancestor model - or an empty list if no such
@@ -107,20 +114,24 @@ def route_to_ancestor(model_class, ancestor_model):
 
     Returns a :class:`list` of :class:`ParentLinkInfo`
     """
-    route = []
+    route: list[ParentLinkInfo] = []
 
-    def find_route(model, target_model, current_route):
+    def find_route(
+        model: type[models.Model],
+        target_model: type[models.Model],
+        current_route: list[ParentLinkInfo],
+    ) -> list[ParentLinkInfo] | None:
         if model is target_model:
             return current_route
 
         for parent_model, field_to_parent in model._meta.parents.items():
             if field_to_parent is not None:
-                new_route = current_route + [ParentLinkInfo(parent_model, field_to_parent)]
+                new_route = current_route + [ParentLinkInfo(parent_model, field_to_parent)]  # type: ignore[arg-type]
                 found_route = find_route(parent_model, target_model, new_route)
                 if found_route is not None:
                     return found_route
             else:
-                return find_route(parent_model, target_model, current_route)
+                return find_route(parent_model, target_model, current_route)  # type: ignore[unreachable]
         return None
 
     found_route = find_route(model_class, ancestor_model, route)
@@ -129,7 +140,7 @@ def route_to_ancestor(model_class, ancestor_model):
     return found_route
 
 
-def is_model_loaded(model):
+def is_model_loaded(model: type[models.Model]) -> bool:
     try:
         apps.get_model(model._meta.app_label, model._meta.model_name)
         return True
@@ -138,7 +149,9 @@ def is_model_loaded(model):
 
 
 @lru_cache(maxsize=None)
-def concrete_descendants(model_class, include_proxy=False):
+def concrete_descendants(
+    model_class: type[models.Model], include_proxy: bool = False
+) -> list[type[models.Model]]:
     """
     Get a list of all concrete (non-abstract, non-proxy) descendant model classes in
     tree order with leaf descendants last. Results are cached.
@@ -147,7 +160,9 @@ def concrete_descendants(model_class, include_proxy=False):
 
     apps.check_models_ready()
 
-    def add_concrete_descendants(model, result):
+    def add_concrete_descendants(
+        model: type[models.Model], result: list[type[models.Model]]
+    ) -> None:
         """Add concrete descendants in tree order (ancestors before descendants)."""
         for sub_cls in model.__subclasses__():
             # Add concrete models in pre-order (parent before children)
@@ -158,12 +173,12 @@ def concrete_descendants(model_class, include_proxy=False):
             # Always recurse to find descendants through abstract and proxy models
             add_concrete_descendants(sub_cls, result)
 
-    result = []
+    result: list[type[models.Model]] = []
     add_concrete_descendants(model_class, result)
     return result
 
 
-def prepare_for_copy(obj):
+def prepare_for_copy(obj: models.Model) -> None:
     """
     Prepare a model instance for copying by resetting all primary keys and parent table
     pointers in the inheritance chain. **Copy semantics are application specific.** This
@@ -230,7 +245,17 @@ def prepare_for_copy(obj):
     obj._state.adding = True  # Mark as new object
 
 
-def _lazy_ctype(model, using=DEFAULT_DB_ALIAS):
+def _lazy_ctype(model: type[models.Model], using: str = DEFAULT_DB_ALIAS) -> ContentType | Q:
+    mgr = ContentType.objects.db_manager(using=using)
+    if apps.models_ready and (
+        cid := mgr._cache.get(using, {}).get((model._meta.app_label, model._meta.model_name))  # type: ignore[attr-defined]
+    ):
+        # cast needed because _cache returns Any
+        return cast(ContentType, cid)
+    return Q(app_label=model._meta.app_label) & Q(model=model._meta.model_name)
+
+
+def lazy_ctype(model: type[models.Model], using: str = DEFAULT_DB_ALIAS) -> ContentType | Subquery:
     """
     Return the content type id for the given model class if it is in the cache,
     otherwise return a subquery that can be used to match the content type as part
@@ -238,17 +263,8 @@ def _lazy_ctype(model, using=DEFAULT_DB_ALIAS):
 
     :param model: The model class to get the content type for.
     :return: The content type for the model class.
-    :rtype: int or Subquery
+    :rtype: ContentType object or Subquery
     """
-    mgr = ContentType.objects.db_manager(using=using)
-    if apps.models_ready and (
-        cid := mgr._cache.get(using, {}).get((model._meta.app_label, model._meta.model_name))
-    ):
-        return cid
-    return Q(app_label=model._meta.app_label) & Q(model=model._meta.model_name)
-
-
-def lazy_ctype(model, using=DEFAULT_DB_ALIAS):
     ctype = _lazy_ctype(model, using=using)
     return (
         ctype
@@ -258,9 +274,9 @@ def lazy_ctype(model, using=DEFAULT_DB_ALIAS):
 
 
 @lru_cache(maxsize=None)
-def _map_queryname_to_class(base_model, qry_name):
+def _map_queryname_to_class(base_model: type[models.Model], qry_name: str) -> type[models.Model]:
     """Try to match a model name in a query to a model class"""
-    name_map = defaultdict(list)
+    name_map: dict[str, list[type[models.Model]]] = defaultdict(list)
     name_map[base_model.__name__.lower()].append(base_model)
     for cls in concrete_descendants(base_model, include_proxy=True):
         name_map[cls.__name__.lower()].append(cls)
@@ -281,7 +297,7 @@ def _map_queryname_to_class(base_model, qry_name):
     raise AssertionError(f"{qry_name} is not a subclass of {base_model._meta.label}")
 
 
-def _clear_utility_caches():
+def _clear_utility_caches() -> None:
     """Clear all lru_cache caches in this module."""
     get_base_polymorphic_model.cache_clear()
     route_to_ancestor.cache_clear()
